﻿
#pragma once
#include <string>
#include <stdint.h>
#include <map>
#include "CATSBTMarcos.h"
#include "CATSBTDataDefine.h"
#include "CATSBTMD.h"

NS_OTP_BEGIN
class ICATSCtx;
class ICATSPrepareCtx;
class CATSBTTickData;
NS_OTP_END



using namespace std;

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

NS_OTP_BEGIN


/************************************************************************/
/* 客户策略需要集成该虚类，并 override 里面的纯虚函数
   
*/
/************************************************************************/
class EXPORT_FLAG ICATSStrategy
{
public:
	virtual ~ICATSStrategy() {}
public:
	
	/************************************************************************/
	/* 系统启动时调用一次，用于初始化客户策略实例， 
	   ICATSPrepareCtx* prepare_ctx：
	   map<string, string> params：自定义参数，可以在客户端”用户参数中设置“，如果名称冲突，则保留最后一个
	   返回值：0-成功，其他：失败。如果失败则策略退出
	*/
	/************************************************************************/
	virtual int init(ICATSPrepareCtx* prepare_ctx, map<string, string> params) { return 0; }

	
	/************************************************************************/
	/*    回测系统已经准备完毕
		  需要将ctx保存下来，后续在回测实例中使用，
	      客户策略需要在该函数中订阅标的，这样框架才会去加载对应标的的tick数据
		  返回值：0-成功，其他：失败。如果失败则策略退出
	*/
	/************************************************************************/
	virtual int on_init(ICATSCtx* ctx) = 0;


	/************************************************************************/
	/* 回测时：每天在碰到第一个tick时触发一次
	   实盘时：系统启动后，当日时间到达start time后（通过 ICATSPrepareCtx.set_start_end_time设置）调用该函数
	   date: YYYYMMDD
	   返回值：0-成功，其他：失败。如果失败则策略退出
	*/	   
	/************************************************************************/
	virtual int on_session_begin(int date) { return 0; }


	/************************************************************************/
	/* 每天执行一次，在改天所有的tick执行完之后触发一次                                                                      
	   对于实盘：当日时间超过end time （通过 ICATSPrepareCtx.set_start_end_time设置）会调用该函数
	   date: YYYYMMDD
	   返回值：0-成功，其他：失败。如果失败则策略退出
	*/
	/************************************************************************/
	virtual int on_session_end(int date) { return 0; }

	
	/************************************************************************/
	/* 整个回测完成之后执行一次                                                                      
	   对于实盘，当日时间超过end time （通过 ICATSPrepareCtx.set_start_end_time设置）先调用 on_session_end，然后调用 on_replay_done
	*/
	/************************************************************************/
	virtual void on_replay_done() {};


	/************************************************************************/
	/*  bar数据回调
	回测时：历史数据
	实盘时：实时行情
	*/
	/************************************************************************/
	virtual void on_bar(CATSBTBarData* bar) {};


	/************************************************************************/
	/*  tick数据回调
	    回测时：历史数据
		实盘时：实时行情
	*/
	/************************************************************************/
	virtual void on_tick(CATSBTTickData* tick) {};

	
	/************************************************************************/
	/* 逐笔成交，该函数只有回测时才有逐笔成交数据
	   实盘时无法获取该数据
	*/
	/************************************************************************/
	virtual void on_transaction(CATSBTTransData* trans) {}


	/************************************************************************/
	/* 订单的成交明细
	   回测：会推送
	   实盘：需要先调用sub_trade进行订阅才会推送，如果没有权限则订阅失败
	*/
	/************************************************************************/
	virtual void on_trade(const TradeData data) {};


	/************************************************************************/
	/* 订单推送
	*/
	/************************************************************************/
	virtual void on_order(const OrderUpdateData data ) {};

	
	/************************************************************************/
	/* 下单后的回调函数
	   is_success:下单成功为true，否则为false，
	   order_no：下单成功时为订单号，失败时该参数无意义
	   message：下单成功时该参数无意义，失败时表示失败原因
	   arg : 下单时传递的arg
	*/
	/************************************************************************/
	virtual void on_entrust(const char* order_no, bool is_success, const char* message, void* arg) {};


	/************************************************************************/
	/*撤单后的回调函数
	  order_no：
	  is_success: 成功或失败，
	  message:成功或失败消息 
	  arg : 撤单时传递的arg
	*/
	/************************************************************************/
	virtual void on_cancel(const char* order_no, bool is_success, const char* message, void* arg) {};
};

NS_OTP_END;

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif


